@@ -5,11 +5,10 @@ from uuid import UUID
 
 import requests.exceptions
 
-import one.alf.folders
 from iblutil.io import hashfile
 
 import one.alf.io as alfio
-from one.alf.files import rel_path_parts, session_path_parts
+from one.alf.files import rel_path_parts, session_path_parts, get_session_path
 import one.alf.exceptions as alferr
 from one.api import ONE
 from one.util import ensure_list
@@ -54,7 +53,7 @@ def register_dataset(file_list, one=None, created_by=None, repository=None, serv
     elif not isinstance(file_list, list):
         file_list = [Path(file_list)]
 
-    assert len(set(alfio.get_session_path(f) for f in file_list)) == 1, 'Multiple sessions in list'
+    assert len(set(get_session_path(f) for f in file_list)) == 1, 'Multiple sessions in list'
 
     assert all([Path(f).exists() for f in file_list])
     if isinstance(versions, str):
@@ -72,7 +71,7 @@ def register_dataset(file_list, one=None, created_by=None, repository=None, serv
     else:
         hashes = [hashfile.md5(p) for p in file_list]
 
-    session_path = alfio.get_session_path(file_list[0])
+    session_path = get_session_path(file_list[0])
 
     # first register the file
     r = {'created_by': created_by,
@@ -176,7 +175,7 @@ class RegistrationClient:
         # Ensure subject exists on Alyx
         self.assert_subject_exists(subject)
         session_root = Path(session_root or self.one.alyx.cache_dir) / subject / f'{date:%Y-%m-%d}'
-        session_path = one.alf.folders.next_num_folder(session_root)
+        session_path = alfio.next_num_folder(session_root)
         eid = UUID(self.create_session(session_path)['url'][-36:]) if register else None
         return session_path, eid
 
