@@ -4,13 +4,14 @@ import urllib.parse
 from functools import wraps
 from typing import Sequence, Union, Iterable, Optional, List
 from collections.abc import Mapping
+import fnmatch
 
 import pandas as pd
 from iblutil.io import parquet
 import numpy as np
 
 import one.alf.exceptions as alferr
-from one.alf.files import rel_path_parts
+from one.alf.files import rel_path_parts, get_session_path
 from one.alf.spec import FILE_SPEC, regex as alf_regex
 import one.alf.io as alfio
 
@@ -45,7 +46,7 @@ def ses2records(ses: dict) -> [pd.Series, pd.DataFrame]:
         rec['eid_0'], rec['eid_1'] = session.name
         file_path = urllib.parse.urlsplit(d['data_url'], allow_fragments=False).path.strip('/')
         file_path = alfio.remove_uuid_file(file_path, dry=True).as_posix()
-        rec['session_path'] = alfio.get_session_path(file_path).as_posix()
+        rec['session_path'] = get_session_path(file_path).as_posix()
         rec['rel_path'] = file_path[len(rec['session_path']):].strip('/')
         if 'default_revision' in d:
             rec['default_revision'] = d['default_revision']
@@ -85,7 +86,7 @@ def datasets2records(datasets) -> pd.DataFrame:
         data_url = urllib.parse.urlsplit(file_record['data_url'], allow_fragments=False)
         file_path = data_url.path.strip('/')
         file_path = alfio.remove_uuid_file(file_path, dry=True).as_posix()
-        rec['session_path'] = alfio.get_session_path(file_path).as_posix()
+        rec['session_path'] = get_session_path(file_path).as_posix()
         rec['rel_path'] = file_path[len(rec['session_path']):].strip('/')
         rec['default_revision'] = d['default_dataset']
         records.append(rec)
@@ -205,7 +206,7 @@ def _collection_spec(collection=None, revision=None):
 
 
 def filter_datasets(all_datasets, filename=None, collection=None, revision=None,
-                    revision_last_before=True, assert_unique=True):
+                    revision_last_before=True, assert_unique=True, wildcards=False):
     """
     Filter the datasets cache table by the relative path (dataset name, collection and revision).
     When None is passed, all values will match.  To match on empty parts, use an empty string.
@@ -215,11 +216,11 @@ def filter_datasets(all_datasets, filename=None, collection=None, revision=None,
     ----------
     all_datasets : pandas.DataFrame
         A datasets cache table
-    filename : str, dict
+    filename : str, dict, None
         A filename str or a dict of alf parts.  Regular expressions permitted.
-    collection : str
+    collection : str, None
         A collection string.  Regular expressions permitted.
-    revision : str
+    revision : str, None
         A revision string to match.  If revision_last_before is true, regular expressions are
         not permitted.
     revision_last_before : bool
@@ -228,6 +229,8 @@ def filter_datasets(all_datasets, filename=None, collection=None, revision=None,
         with regular expressions permitted.
     assert_unique : bool
         When true an error is raised if multiple collections or datasets are found
+    wildcards : bool
+        If true, use unix shell style matching instead of regular expressions
 
     Returns
     -------
@@ -254,12 +257,22 @@ def filter_datasets(all_datasets, filename=None, collection=None, revision=None,
         spec_str += FILE_SPEC
         regex_args.update(**filename)
     else:
-        spec_str += filename + '$'  # Assert end of string
+        # Convert to regex is necessary and assert end of string
+        spec_str += fnmatch.translate(filename) if wildcards else filename + '$'
 
     # If matching revision name, add to regex string
     if not revision_last_before:
         regex_args.update(revision=revision)
 
+    for k, v in regex_args.items():
+        if v is None:
+            continue
+        if wildcards:
+            # Convert to regex, remove \\Z which asserts end of string
+            v = (fnmatch.translate(x).replace('\\Z', '') for x in ensure_list(v))
+        if not isinstance(v, str):
+            regex_args[k] = '|'.join(v)  # logical OR
+
     # Build regex string
     pattern = alf_regex('^' + spec_str, **regex_args)
     match = all_datasets[all_datasets['rel_path'].str.match(pattern)]
